-- Создание таблицы Научных конференций
create table config
  (CID number(5),                --идентификатор конференции
   LECTURE_TITLE varchar2(255) NOT NULL, --название конференции
   LECTURE_THEME varchar2(255) NOT NULL, --тема конференции
   LECTURE_DATE date,                   --дата проведения
   NUMconfig number(5)             --число участников
   );

-- Создание таблицы Пленарных докладов
create table lectures
  (LID number(5),                --идентификатор доклада
   LTHEME varchar2(255) NOT NULL, --Доклад №
   AUTHOR varchar2(255) NOT NULL, --автор
   AUTHOR_RANK varchar2(255) NOT NULL, --ученое звание
   CID number(5)                 --идентификатор конференции
   );

--Определение первичного ключа таблицы config
ALTER TABLE config
   ADD (CONSTRAINT config_pk_cid PRIMARY KEY (cid));

--Определение первичного и внешнего ключей таблицы lectures
ALTER TABLE lectures
   ADD (CONSTRAINT lectures_pk_lid PRIMARY KEY (lid),
        CONSTRAINT lectures_fk_cid FOREIGN KEY (cid)
        REFERENCES config(cid));

--Триггер для таблицы lectures, запрещающий изменение во внерабочее время
CREATE OR REPLACE TRIGGER Tr_lectures
BEFORE INSERT OR UPDATE OR DELETE ON lectures
FOR EACH ROW
WHEN (TO_CHAR(SYSDATE, 'D') > 5 OR
     (TO_CHAR(SYSDATE, 'HH24') < 8 OR
	 TO_CHAR(SYSDATE, 'HH24') > 16))
BEGIN
  raise_application_error(-20001, 'Запрещены изменения во внерабочее время');
END Tr_lectures;
/

--Представление выводящее первые 5 докладов по дате конференций
CREATE VIEW First_5_lectures AS
  SElectures * FROM
    (SElectures lecture_date, author, ltheme
     FROM config, lectures
     WHERE config.cid = lectures.cid
     ORDER BY lecture_date)
  WHERE ROWNUM <= 5;

--Создание пакета процедур
CREATE OR REPLACE PACKAGE ProceduresPack IS
  PROCEDURE TabFill;        --Процедура начального заполнения таблиц
  PROCEDURE TabClear;       --Процедура очистки таблиц
  PROCEDURE ShiftDays(num IN NUMBER); --Процедура сдвига даты проведения конференции
  PROCEDURE Printconfig(auth_tit IN VARCHAR2); --Вывод конференций по званию автора, переданного в параметре
END ProceduresPack;
/

--Создание тела пакета процедур
CREATE OR REPLACE PACKAGE BODY ProceduresPack IS
  --Процедура начального заполнения таблиц
  PROCEDURE TabFill IS
  BEGIN
  --
  --заполнение таблицы config
  --
    INSERT INTO config
      VALUES(100, 'III Международная научно-практическая конференция', 'Актуальные проблемы медицины в России и за рубежом', to_date('01.04.2016','dd.mm.yyyy'), 89);
    INSERT INTO config
      VALUES(101, 'XLIX Международная научно-практическая конференция', 'Актуальные вопросы экономических наук', to_date('23.04.2016','dd.mm.yyyy'), 23);
    INSERT INTO config
      VALUES(102, 'XXX  Международная научно-практическая конференция', 'Проблемы современной экономики', to_date('16.05.2016','dd.mm.yyyy'), 19);
    INSERT INTO config
      VALUES(103, 'XXIX Международная научно-практическая конференция', 'Перспективы развития информационных технологий', to_date('17.06.2016','dd.mm.yyyy'), 280);
    INSERT INTO config
      VALUES(104, 'XLIV Международная научно-практическая конференция', 'Наука и современность - 2016', to_date('18.07.2016','dd.mm.yyyy'), 77);
  --
  --заполнение таблицы lectures
  --
    INSERT INTO lectures
      VALUES(200, 'Доклад № 1', 'Иванов И.И.', 'профессор', 100);
    INSERT INTO lectures
      VALUES(201, 'Доклад № 2', 'Смирнов С.С.', 'профессор', 101);
    INSERT INTO lectures
      VALUES(202, 'Доклад № 3', 'Сидоров С.С.', 'доцент', 102);
    INSERT INTO lectures
      VALUES(203, 'Доклад № 4', 'Петросян Е.В.', 'профессор', 103);
    INSERT INTO lectures
      VALUES(204, 'Доклад № 5', 'Петросян Е.И.', 'профессор', 104);
    INSERT INTO lectures
      VALUES(205, 'Доклад № 6', 'Сидоров В.В.', 'доцент', 100);
    INSERT INTO lectures
      VALUES(206, 'Доклад № 7', 'Иванов И.И.', 'профессор', 101);
    INSERT INTO lectures
      VALUES(207, 'Доклад № 8', 'Соломкин Е.С.', 'доцент', 102);
    INSERT INTO lectures
      VALUES(208, 'Доклад № 9', 'Попов И.И.', 'профессор', 103);
    INSERT INTO lectures
      VALUES(209, 'Доклад № 10', 'Лаптев В.В.', 'доцент', 104);

    COMMIT;

    EXCEPTION
	
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE('Ошибка при заполнении');

  END TabFill;

  --Процедура очистки таблиц
  PROCEDURE TabClear IS
  BEGIN
    DELETE FROM lectures;
    DELETE FROM config;
  END TabClear;
  
  --Процедура сдвига даты проведения конференции
  PROCEDURE ShiftDays(num IN NUMBER) IS
  BEGIN
    UPDATE config SET lecture_date = lecture_date + num
	WHERE numconfig < (SElectures MAX(numconfig) FROM config);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE('Ошибка при переносе дат');
  END ShiftDays;
  
  --Процедура вывода конференций по званию автора, переданного в параметре
  PROCEDURE Printconfig(auth_tit IN VARCHAR2) IS
  
  CURSOR curs(auth_tit VARCHAR2) 	IS
    SElectures config.*, author, AUTHOR_RANK, ltheme FROM  config, lectures
	WHERE config.cid=lectures.cid
	AND AUTHOR_RANK=auth_tit
	ORDER BY lecture_date;
	
	cid NUMBER(4) := 0;
	
  BEGIN
    FOR r IN curs(auth_tit) LOOP
	  IF cid <> r.cid THEN
	    DBMS_OUTPUT.PUT_LINE(r.lecture_date||'  '||r.lecture_title||'  '||r.lecture_theme);
	  END IF;
	  DBMS_OUTPUT.PUT_LINE('  '||r.author||'  '||r.AUTHOR_RANK||'  '||r.ltheme);
	  cid := r.cid;
	END LOOP;
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE('Ошибка процедуры Printconfig!');
  END Printconfig;

END ProceduresPack;
/